@@ -15,7 +15,7 @@ print "\nRunning on a ",Sys::CPU::cpu_type(), " with $Threads threads\n";
 sleep 3;
 my @framebuffer;
 foreach my $thr (0..$Threads) {
-    $framebuffer[$thr] = Graphics::Framebuffer->new('FB_DEVICE' => '/dev/fb0');
+    $framebuffer[$thr] = Graphics::Framebuffer->new('FB_DEVICE' => '/dev/fb0','FILE_MODE' => 0);
 }
 my $screen_width  = $framebuffer[0]->{'XRES'};
 my $screen_height = $framebuffer[0]->{'YRES'};
