@@ -7,12 +7,12 @@ use Graphics::Framebuffer;
 use Term::ANSIScreen qw(:cursor);
 
 $| = 1;
-my $F = Graphics::Framebuffer->new('FB_DEVICE' => '/dev/fb0');
+my $F = Graphics::Framebuffer->new('FB_DEVICE' => '/dev/fb0','FILE_MODE' => 0);
 
 my $screen_width  = $F->{'XRES'};
 my $screen_height = $F->{'YRES'};
-my $XX            = $screen_width - 1;
-my $YY            = $screen_height - 1;
+my $XX            = $screen_width;
+my $YY            = $screen_height;
 my $center_x      = int($screen_width / 2);
 my $center_y      = int($screen_height / 2);
 my $wait          = 2;
@@ -73,9 +73,7 @@ $F->cls();
 print locate(1,1),'Testing fill                               ';
 $F->polygon({'coordinates' => [0,20,$XX,20,$center_x+200,$YY,$center_x,$center_y,$center_x-200,$YY]});
 $F->fill({'x' => 100, 'y' => 100});
-sleep $wait;
+# sleep $wait;
 
 __END__
 
-__C__
-
