@@ -84,7 +84,7 @@ use constant {
 use 5.010;
 
 use Switch;          # Yes, a touch of new Perl. Switch is so much nicer
-                     # a ton of if-else's
+                     # than a ton of if-else's
 use Math::Trig qw(:pi);    # Yes, yummy PI
 use Sys::Mmap;             # Absolutely necessary to map the screen to a string.
                            # Without this, I doubt this module would exist.
@@ -92,11 +92,13 @@ use Sys::Mmap;             # Absolutely necessary to map the screen to a string.
 use Imager;                # This is used for TrueType font printing, and where
                            # 'Pure Perl' goes up in smoke.
 
+# use Data::Dumper::Simple;
+
 BEGIN {
     require Exporter;
 
     # set the version for version checking
-    our $VERSION   = 4.21;
+    our $VERSION   = 4.22;
     our @ISA       = qw(Exporter);
     our @EXPORT    = qw();
     our @EXPORT_OK = qw();
@@ -127,6 +129,13 @@ my $fb = Graphics::Framebuffer->new(parameter => value);
 
  Framebuffer device name. Default is /dev/fb0
 
+=item B< FILE_MODE >
+
+ Sets the internal drawing system to use file handle mode
+ instead of memory mapped mode.  File mode is more stable,
+ but a bit slower.  I recommend doing this ONLY if you
+ are having issues with memory mapped mode.
+
 =item B< COLOR_ORDER >
 
 Sets the coloring logic to use the proper color ordering.
@@ -183,6 +192,7 @@ sub new {
         'ARC'         => 0,
         'PIE'         => 1,
         'POLY_ARC'    => 2,
+        'FILE_MODE'   => FALSE,
 
         ## Set up the Framebuffer driver "constants" defaults
         # Commands
@@ -259,7 +269,7 @@ sub new {
 
         # Now that everything is set up, let's map the framebuffer to SCREEN
 
-        mmap($self->{'SCREEN'},  $self->{'smem_len'}, PROT_READ | PROT_WRITE, MAP_SHARED, $self->{'FB'});
+        mmap($self->{'SCREEN'},  $self->{'smem_len'}, PROT_READ | PROT_WRITE, MAP_SHARED, $self->{'FB'}) unless($self->{'FILE_MODE'});
     } else {    # Go into emulation mode if no actual framebuffer available
                 # Aww bummer, no screen.  Ok, turn on emulation mode then
         $self->{'ERROR'} = 'Framebuffer Device Not Found! Emulation mode.  EXPERIMENTAL!!';
@@ -268,6 +278,8 @@ sub new {
         $self->{'SCREEN'}         = chr(0) x ($self->{'VXRES'} * $self->{'VYRES'} * $self->{'BYTES'});
         $self->{'XRES'}           = $self->{'VXRES'};                                                                                                          # Virtual and physical are the same
         $self->{'YRES'}           = $self->{'VYRES'};
+        $self->{'XOFFSET'}        = 0;
+        $self->{'YOFFSET'}        = 0;
         $self->{'PIXELS'}         = (($self->{'XOFFSET'} + $self->{'VXRES'}) * ($self->{'YOFFSET'} + $self->{'VYRES'}));
         $self->{'SIZE'}           = $self->{'PIXELS'} * $self->{'BYTES'};
         $self->{'smem_len'}       = $self->{'BYTES'} * ($self->{'VXRES'} * $self->{'VYRES'}) if (!defined($self->{'smem_len'}) || $self->{'smem_len'} <= 0);
@@ -296,7 +308,7 @@ $fb->screen_close();
 sub screen_close {
     my $self = shift;
     if (!defined($self->{'ERROR'})) {    # Only do it if not in emulation mode
-        munmap($self->{'SCREEN'}) if (defined($self->{'SCREEN'}));
+        munmap($self->{'SCREEN'}) if (defined($self->{'SCREEN'}) && !$self->{'FILE_MODE'});
         close($self->{'FB'}) if (defined($self->{'FB'}));
         delete($self->{'FB'});           # We leave no remnants MUHWAHAHAHAHA!!!
     }
@@ -525,6 +537,7 @@ sub plot {
     my $self   = shift;
     my $params = shift;
 
+    my $fb   = $self->{'FB'};
     my $x    = int($params->{'x'}          || 0);    # Ignore decimals
     my $y    = int($params->{'y'}          || 0);
     my $size = int($params->{'pixel_size'} || 1);
@@ -550,7 +563,12 @@ sub plot {
                 $index = ($self->{'BYTES_PER_LINE'} * ($y + $self->{'YOFFSET'})) + ($x * $self->{'BYTES'});
                 if ($index >= 0 && $index <= ($self->{'smem_len'} - $self->{'BYTES'})) {
                     eval {
-                        $c = substr($self->{'SCREEN'}, $index, $self->{'BYTES'}) || chr(0)x$self->{'BYTES'};
+                        if ($self->{'FILE_MODE'}) {
+                            seek($fb,$index,0);
+                            read($fb,$c,$self->{'BYTES'});
+                        } else {
+                            $c = substr($self->{'SCREEN'}, $index, $self->{'BYTES'}) || chr(0)x$self->{'BYTES'};
+                        }
                         switch ($self->{'DRAW_MODE'}) {
                             case ($self->{'NORMAL_MODE'}) {
                                 $c = $self->{'COLOR'};
@@ -571,7 +589,12 @@ sub plot {
                                 $c = $self->{'COLOR'} if ($self->pixel($x, $y) eq $self->{'B_COLOR'});
                             }
                         } ## end switch
-                        substr($self->{'SCREEN'}, $index, $self->{'BYTES'}) = $c;
+                        if ($self->{'FILE_MODE'}) {
+                            seek($fb,$index,0);
+                            print $fb $c;
+                        } else {
+                            substr($self->{'SCREEN'}, $index, $self->{'BYTES'}) = $c;
+                        }
                     };
                     $self->_fix_mapping() if ($@);
                 } ## end if ($index >= 0 && $index...)
@@ -1466,6 +1489,7 @@ sub pixel {
     my $self   = shift;
     my $params = shift;
 
+    my $fb = $self->{'FB'};
     my $x = int($params->{'x'});
     my $y = int($params->{'y'});
 
@@ -1473,7 +1497,12 @@ sub pixel {
     unless (($x > $self->{'XX_CLIP'}) || ($y > $self->{'YY_CLIP'}) || ($x < $self->{'X_CLIP'}) || ($y < $self->{'Y_CLIP'})) {
         my ($color, $R, $G, $B, $A);
         my $index = ($self->{'BYTES_PER_LINE'} * ($y + $self->{'YOFFSET'})) + ($x * $self->{'BYTES'});
-        $color = substr($self->{'SCREEN'}, $index, $self->{'BYTES'});
+        if ($self->{'FILE_MODE'}) {
+            seek($fb,$index,0);
+            read($fb,$color,$self->{'BYTES'});
+        } else {
+            $color = substr($self->{'SCREEN'}, $index, $self->{'BYTES'});
+        }
         if ($self->{'BITS'} == 32) {
             unless ($self->{'COLOR_ORDER'}) {    # BGR
                 ($B, $G, $R, $A) = unpack('C4', $color);
@@ -1685,6 +1714,7 @@ sub blit_read {
     my $self   = shift;
     my $params = shift;
 
+    my $fb = $self->{'FB'};
     my $x = int($params->{'x'});
     my $y = int($params->{'y'});
     my $w = int($params->{'width'});
@@ -1701,7 +1731,14 @@ sub blit_read {
     my ($index, $scrn, $line);
     for ($line = $y ; $line < $yend ; $line++) {
         $index = ($self->{'BYTES_PER_LINE'} * ($line + $self->{'YOFFSET'})) + $XX;
-        $scrn .= substr($self->{'SCREEN'}, $index, $W);
+        if ($self->{'FILE_MODE'}) {
+            seek($fb,$index,0);
+            my $buf;
+            read($fb,$buf,$W);
+            $scrn .= $buf;
+        } else {
+            $scrn .= substr($self->{'SCREEN'}, $index, $W);
+        }
     }
 
     return ({'x' => $x, 'y' => $y, 'width' => $w, 'height' => $h, 'image' => $scrn});
@@ -1730,6 +1767,7 @@ sub blit_write {
     my $self   = shift;
     my $params = shift;
 
+    my $fb   = $self->{'FB'};
     my $x    = int($params->{'x'});
     my $y    = int($params->{'y'});
     my $w    = int($params->{'width'}) || 1;
@@ -1777,29 +1815,76 @@ sub blit_write {
             if ($index >= 0 && $index <= $max && $idx >= 0 && $idx <= (length($scrn) - $self->{'BYTES'})) {
                 switch ($self->{'DRAW_MODE'}) {
                     case ($self->{'NORMAL_MODE'}) {
-                        substr($self->{'SCREEN'}, $index, $scan) = substr($scrn, $idx, $scan);
+                        if ($self-{'FILE_MODE'}) {
+                            seek($fb,$index,0);
+                            print $fb substr($scrn,$idx,$scan);
+                        } else {
+                            substr($self->{'SCREEN'}, $index, $scan) = substr($scrn, $idx, $scan);
+                        }
                     }
                     case ($self->{'XOR_MODE'}) {
-                        substr($self->{'SCREEN'}, $index, $scan) ^= substr($scrn, $idx, $scan);
+                        if ($self-{'FILE_MODE'}) {
+                            seek($fb,$index,0);
+                            my $buf = '';
+                            read($fb,$buf,$scan);
+                            substr($buf,0,$scan) ^= substr($scrn,$idx,$scan);
+                            seek($fb,$index,0);
+                            print $fb $buf;
+                        } else {
+                            substr($self->{'SCREEN'}, $index, $scan) ^= substr($scrn, $idx, $scan);
+                        }
                     }
                     case ($self->{'OR_MODE'}) {
-                        substr($self->{'SCREEN'}, $index, $scan) |= substr($scrn, $idx, $scan);
+                        if ($self-{'FILE_MODE'}) {
+                            seek($fb,$index,0);
+                            my $buf = '';
+                            read($fb,$buf,$scan);
+                            substr($buf,0,$scan) |= substr($scrn,$idx,$scan);
+                            seek($fb,$index,0);
+                            print $fb $buf;
+                        } else {
+                            substr($self->{'SCREEN'}, $index, $scan) |= substr($scrn, $idx, $scan);
+                        }
                     }
                     case ($self->{'AND_MODE'}) {
-                        substr($self->{'SCREEN'}, $index, $scan) &= substr($scrn, $idx, $scan);
+                        if ($self-{'FILE_MODE'}) {
+                            seek($fb,$index,0);
+                            my $buf = '';
+                            read($fb,$buf,$scan);
+                            substr($buf,0,$scan) &= substr($scrn,$idx,$scan);
+                            seek($fb,$index,0);
+                            print $fb $buf;
+                        } else {
+                            substr($self->{'SCREEN'}, $index, $scan) &= substr($scrn, $idx, $scan);
+                        }
                     }
                     case ($self->{'MASK_MODE'}) {
                         for ($px = 0 ; $px < $w ; $px++) {
                             unless ($px > $self->{'XX_CLIP'} || $px < $self->{'X_CLIP'}) {
                                 $px4 = $px * $self->{'BYTES'};
-                                $data = substr($self->{'SCREEN'}, ($index + $px4), $self->{'BYTES'}) || chr(0)x$self->{'BYTES'};
+                                if ($self-{'FILE_MODE'}) {
+                                    seek($fb,$index,0);
+                                    read($fb,$data,$self->{'BYTES'});
+                                } else {
+                                    $data = substr($self->{'SCREEN'}, ($index + $px4), $self->{'BYTES'}) || chr(0)x$self->{'BYTES'};
+                                }
                                 if ($self->{'BITS'} == 32) {
                                     if (substr($scrn, ($idx + $px4), 3) . chr(255) ne $self->{'B_COLOR'}) {
-                                        substr($self->{'SCREEN'}, ($index + $px4), $self->{'BYTES'}) = substr($scrn, ($idx + $px4), $self->{'BYTES'});
+                                        if ($self-{'FILE_MODE'}) {
+                                            seek($fb,$index + $px4, 0);
+                                            print $fb substr($scrn, ($idx + $px4), $self->{'BYTES'});
+                                        } else {
+                                            substr($self->{'SCREEN'}, ($index + $px4), $self->{'BYTES'}) = substr($scrn, ($idx + $px4), $self->{'BYTES'});
+                                        }
                                     }
                                 } else {
                                     if (substr($scrn, ($idx + $px4), 2) ne $self->{'B_COLOR'}) {
-                                        substr($self->{'SCREEN'}, ($index + $px4), $self->{'BYTES'}) = substr($scrn, ($idx + $px4), $self->{'BYTES'});
+                                        if ($self-{'FILE_MODE'}) {
+                                            seek($fb,$index + $px4, 0);
+                                            print $fb substr($scrn, ($idx + $px4), $self->{'BYTES'});
+                                        } else {
+                                            substr($self->{'SCREEN'}, ($index + $px4), $self->{'BYTES'}) = substr($scrn, ($idx + $px4), $self->{'BYTES'});
+                                        }
                                     }
                                 }
                             } ## end if ($px <= $self->{'XX_CLIP'...})
@@ -1809,14 +1894,29 @@ sub blit_write {
                         for ($px = 0 ; $px < $w ; $px++) {
                             unless ($px > $self->{'XX_CLIP'} || $px < $self->{'X_CLIP'}) {
                                 $px4 = $px * $self->{'BYTES'};
-                                $data = substr($self->{'SCREEN'}, ($index + $px4), $self->{'BYTES'});
+                                if ($self-{'FILE_MODE'}) {
+                                    seek($fb,$index + $px4, 0);
+                                    read($fb,$data,$self->{'BYTES'});
+                                } else {
+                                    $data = substr($self->{'SCREEN'}, ($index + $px4), $self->{'BYTES'});
+                                }
                                 if ($self->{'BITS'} == 32) {
                                     if (substr($self->{'SCREEN'}, ($index + $px4), 3) . chr(255) eq $self->{'B_COLOR'}) {
-                                        substr($self->{'SCREEN'}, ($index + $px4), $self->{'BYTES'}) = substr($scrn, ($idx + $px4), $self->{'BYTES'});
+                                        if ($self-{'FILE_MODE'}) {
+                                            seek($fb,$index+$px4,0);
+                                            print $fb substr($scrn,$idx+$px4,$self->{'BYTES'});
+                                        } else {
+                                            substr($self->{'SCREEN'}, ($index + $px4), $self->{'BYTES'}) = substr($scrn, ($idx + $px4), $self->{'BYTES'});
+                                        }
                                     }
                                 } else {
                                     if (substr($self->{'SCREEN'}, ($index + $px4), 2) eq $self->{'B_COLOR'}) {
-                                        substr($self->{'SCREEN'}, ($index + $px4), $self->{'BYTES'}) = substr($scrn, ($idx + $px4), $self->{'BYTES'});
+                                        if ($self-{'FILE_MODE'}) {
+                                            seek($fb,$index+$px4,0);
+                                            print $fb substr($scrn,$idx+$px4,$self->{'BYTES'});
+                                        } else {
+                                            substr($self->{'SCREEN'}, ($index + $px4), $self->{'BYTES'}) = substr($scrn, ($idx + $px4), $self->{'BYTES'});
+                                        }
                                     }
                                 }
                             } ## end if ($px <= $self->{'XX_CLIP'...})
@@ -2296,8 +2396,11 @@ sub RGB_to_RGBA {
 
 sub _fix_mapping {
     my $self = shift;
-    munmap($self->{'SCREEN'});
-    mmap($self->{'SCREEN'}, $self->{'smem_len'}, PROT_READ | PROT_WRITE, MAP_SHARED, $self->{'FB'});
+    unless($self->{'FILE_MODE'}) {
+        my $fb = $self->{'FB'};
+        munmap($self->{'SCREEN'});
+        mmap($self->{'SCREEN'}, $self->{'smem_len'}, PROT_READ | PROT_WRITE, MAP_SHARED, $fb);
+    }
 }
 ## Just standard flat subroutines
 
@@ -2383,7 +2486,7 @@ modify it under the same terms as Perl itself.
 
 =head1 VERSION
 
-Version 4.21 (July 16, 2015)
+Version 4.22 (July 21, 2015)
 
 =head1 THANKS
 
